==> vdocipher_flutter-2.4.2.tar.gz: lib/src/web/vdo_web_player_controller.dart <==
import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:vdocipher_flutter/src/web/vdo_player_platform_interface.dart';
import 'package:vdocipher_flutter/vdocipher_flutter.dart';

VideoPlayerPlatform? _lastVideoPlayerPlatform;

VideoPlayerPlatform get videoPlayerPlatform {
  final VideoPlayerPlatform currentInstance = VideoPlayerPlatform.instance;
  if (_lastVideoPlayerPlatform != currentInstance) {
    // This will clear all open videos on the platform when a full restart is
    // performed.
    currentInstance.init();
    _lastVideoPlayerPlatform = currentInstance;
  }
  return currentInstance;
}

typedef PlatformViewCreatedCallback = void Function(int id);

class _VideoAppLifeCycleObserver extends Object with WidgetsBindingObserver {
  _VideoAppLifeCycleObserver(this._controller);

  bool _wasPlayingBeforePause = false;
  final WebPlayerController _controller;

  void initialize() {
    _ambiguate(WidgetsBinding.instance)!.addObserver(this);
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    if (state == AppLifecycleState.paused) {
      _wasPlayingBeforePause = _controller.value.isPlaying;
      _controller.pause();
    } else if (state == AppLifecycleState.resumed) {
      if (_wasPlayingBeforePause) {
        _controller.play();
      }
    }
  }

  void dispose() {
    _ambiguate(WidgetsBinding.instance)!.removeObserver(this);
  }
}

/// Controls a platform video player, and provides updates when the state is
/// changing.
///
/// Instances must be initialized with initialize.
///
/// To reclaim the resources used by the player call [dispose].
///
/// After [dispose] all further calls are ignored.
class WebPlayerController extends VdoPlayerController {
  _VideoAppLifeCycleObserver? _lifeCycleObserver;
  StreamSubscription<dynamic>? _eventSubscription;
  final List<PlatformViewCreatedCallback> _platformViewCreatedCallbacks =
      <PlatformViewCreatedCallback>[];

  /// The id of a texture that hasn't been initialized.
  static const int kUninitializedTextureId = -1;
  int _textureId = kUninitializedTextureId;
  bool _isDisposed = false;

  WebPlayerController() : super();

  void addPlatformViewCreatedCallback(PlatformViewCreatedCallback listener) {
    if (!_isDisposed) {
      _platformViewCreatedCallbacks.add(listener);
    }
  }

  void removeOnPlatformViewCreatedListener(
      PlatformViewCreatedCallback listener) {
    if (!_isDisposed) {
      _platformViewCreatedCallbacks.remove(listener);
    }
  }

  int get textureId => _textureId;

  @override
  void init() {
    WidgetsBinding.instance.addObserver(this);
  }

  Future<void> initialize(EmbedInfo embedInfo, bool showControls) async {
    init();
    _lifeCycleObserver = _VideoAppLifeCycleObserver(this);
    _lifeCycleObserver?.initialize();

    _textureId = (await videoPlayerPlatform.create(embedInfo, showControls)) ??
        kUninitializedTextureId;

    for (PlatformViewCreatedCallback platformViewCreatedCallback
        in _platformViewCreatedCallbacks) {
      platformViewCreatedCallback(_textureId);
    }

    final Completer<void> initializingCompleter = Completer<void>();

    void eventListener(VideoEvent event) {
      if (_isDisposed) {
        return;
      }

      switch (event.eventType) {
        case VideoEventType.initialized:
          if (embedInfo.embedInfoOptions?.autoplay ?? false) {
            value.copyWith(isPlaying: true);
            notifyListeners();
            funStartPositionTimer?.call();
          }
          break;
        case VideoEventType.apiReady:
          break;
        case VideoEventType.play:
          value = value.copyWith(isPlaying: true, isBuffering: false);
          notifyListeners();
          break;
        case VideoEventType.playing:
          value = value.copyWithoutError(
              isPlaying: true,
              isBuffering: false,
              isLoading: false,
              isEnded: false);
          notifyListeners();
          break;
        case VideoEventType.waiting:
          value = value.copyWith(isBuffering: true, isPlaying: false);
          notifyListeners();
          break;
        case VideoEventType.pause:
          value = value.copyWith(isPlaying: false);
          notifyListeners();
          break;
        case VideoEventType.loadeddata:
          value = value.copyWith(isLoading: false);
          if (event.duration != null) {
            value = value.copyWith(duration: event.duration);
            notifyListeners();
          }
          _updateVideoTracks();
          _updateSubtitleTracks();
          _updateVolume();
          break;
        case VideoEventType.loadstart:
          videoPlayerPlatform.updatePlayerConfigMap(
              textureId, embedInfo.embedInfoOptions?.configMap);
          value = value.copyWith(isLoading: true);
          break;
        case VideoEventType.ratechange:
          if (event.playbackRate != null) {
            value = value.copyWith(playbackSpeed: event.playbackRate);
            notifyListeners();
          }
          break;
        case VideoEventType.progress:
          if (event.progress != null) {
            value = value.copyWith(position: event.progress);
            notifyListeners();
          }
          break;
        case VideoEventType.error:
          if (event.error != null) {
            Error error = event.error!;
            VdoError vdoError = VdoError(error.code!, error.message ?? "", -1);
            value = value.copyWith(vdoError: vdoError);
          }
          break;
        case VideoEventType.bufferingStart:
          break;
        case VideoEventType.bufferingEnd:
          break;
        case VideoEventType.bufferingUpdate:
          if (event.bufferedPosition != null) {
            value = value.copyWith(bufferedPosition: event.bufferedPosition);
            notifyListeners();
          }
          break;
        case VideoEventType.ended:
          value = value.copyWith(isEnded: true);
          notifyListeners();
          break;
        case VideoEventType.volumechange:
          if (event.volume != null) {
            value =
                value.copyWith(volume: event.volume, isMuted: event.isMuted);
            notifyListeners();
          }
          break;
        case VideoEventType.captionLanguageChange:
          if (event.textTrackId != null) {
            for (SubtitleTrack subtitleTrack in value.subtitleTracks) {
              if (subtitleTrack.id == event.textTrackId) {
                value = value.copyWith(subtitleTrack: subtitleTrack);
                break;
              }
            }
            notifyListeners();
          }
          break;
        case VideoEventType.videoQualityChange:
          if (event.videoTrackId != null) {
            for (VideoTrack videoTrack in value.videoTracks) {
              if (videoTrack.id == event.videoTrackId) {
                value = value.copyWith(videoTrack: videoTrack);
                break;
              }
            }
            notifyListeners();
          }
          break;
        case VideoEventType.videoAdaptivenessChange:
          value = value.copyWith(isAdaptive: event.isAdaptive);
          notifyListeners();
          break;
        case VideoEventType.captionVisibilityChange:
          if (!event.isCaptionEnabled) {
            value = value.copyWithDisabledSubtitle(subtitleTrack: null);
            notifyListeners();
          }
          break;
        case VideoEventType.unknown:
          break;
      }
    }

    void errorListener(Object obj) {
      final PlatformException e = obj as PlatformException;
      debugPrint("load error [ ${e.code} , ${e.message!} + ]");
      String errMsg = e.message ?? "unknown error";
      VdoError error = VdoError(int.parse(e.code), errMsg, null);
      value =
          value.copyWith(isLoading: false, isBuffering: false, vdoError: error);
    }

    _eventSubscription = videoPlayerPlatform
        .videoEventsFor(_textureId)
        .listen(eventListener, onError: errorListener);

    return initializingCompleter.future;
  }

  @override
  void dispose() {
    super.dispose();
    funStopPositionTimer?.call();
    disposeWebController();
  }

  Future<void> disposeWebController() async {
    if (_isDisposed) {
      return;
    }
    if (!_isDisposed) {
      _isDisposed = true;
      await _eventSubscription?.cancel();
      await videoPlayerPlatform.dispose(_textureId);
      _platformViewCreatedCallbacks.clear();
      WidgetsBinding.instance.removeObserver(this);
    }
    _lifeCycleObserver?.dispose();
    _isDisposed = true;
  }

  @override
  Future<void> load(EmbedInfo embedInfo) async {
    if (embedInfo.otp != null && embedInfo.playbackInfo != null) {
      await videoPlayerPlatform.load(
          textureId, embedInfo.otp!, embedInfo.playbackInfo!);
      if (embedInfo.embedInfoOptions?.autoplay ?? false) {
        value.copyWith(isPlaying: true);
        notifyListeners();
        if (value.isPlaying) {
          funStartPositionTimer?.call();
        }
      }
    }
  }

  @override
  Future<void> play() async {
    value = value.copyWith(isPlaying: true);
    funStartPositionTimer?.call();
    await videoPlayerPlatform.play(textureId);
    notifyListeners();
  }

  @override
  Future<void> pause() async {
    value = value.copyWith(isPlaying: false);
    await videoPlayerPlatform.pause(textureId);
    notifyListeners();
  }

  @override
  Future<void> stop() async {}

  @override
  Future<Duration> getDuration() async {
    return videoPlayerPlatform.getDuration(textureId);
  }

  @override
  Future<Duration> getPosition() async {
    return videoPlayerPlatform.getPosition(textureId);
  }

  @override
  Future<Duration> getBufferedPosition() {
    return videoPlayerPlatform.getBufferedPosition(textureId);
  }

  @override
  Future<void> setVolume(double volume) {
    return videoPlayerPlatform.setVolume(textureId, volume);
  }

  @override
  Future<void> setMute(bool mute) {
    return videoPlayerPlatform.setMute(textureId, mute);
  }

  @override
  Future<void> seek(Duration target) async {
    target = target > value.duration ? value.duration : target;
    return videoPlayerPlatform.seekTo(textureId, target);
  }

  @override
  Future<void> setPlaybackSpeed(double speed) async {
    value = value.copyWith(playbackSpeed: speed);
    await videoPlayerPlatform.setPlaybackSpeed(textureId, 2);
    notifyListeners();
  }

  @override
  Future<void> setAdaptive() async {
    return videoPlayerPlatform.setAdaptive(textureId);
  }

  @override
  Future<Object> getPlaybackProperty(String propertyName) {
    return videoPlayerPlatform.getPlaybackProperty(textureId, propertyName);
  }

  void _updateVideoTracks() async {
    List<VideoTrack> videoTracks =
        await videoPlayerPlatform.getVideoTracks(textureId);
    VideoTrack? videoTrack = await videoPlayerPlatform.getVideoTrack(textureId);
    value = value.copyWith(videoTracks: videoTracks, videoTrack: videoTrack);
    notifyListeners();
  }

  void _updateSubtitleTracks() async {
    List<SubtitleTrack> subtitleTracks =
        await videoPlayerPlatform.getSubtitleTracks(textureId);
    SubtitleTrack? subtitleTrack =
        await videoPlayerPlatform.getSubtitleTrack(textureId);
    value = value.copyWith(
        subtitleTracks: subtitleTracks, subtitleTrack: subtitleTrack);
    notifyListeners();
  }

  void _updateVolume() async {
    double volume = await videoPlayerPlatform.getVolume(textureId);
    value = value.copyWith(volume: volume);
    notifyListeners();
  }

  @override
  Future<void> setVideoTrack(VideoTrack videoTrack) async {
    return videoPlayerPlatform.setVideoTrackId(textureId, videoTrack.id);
  }

  @override
  Future<void> setSubtitleLanguage(String? language) async {
    return videoPlayerPlatform.setTextTrackLanguage(textureId, language);
  }
}

T? _ambiguate<T>(T? value) => value;

==> vdocipher_flutter-2.4.4.tar.gz: lib/src/web/vdo_web_player_controller.dart <==
import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:vdocipher_flutter/src/web/vdo_player_platform_interface.dart';
import 'package:vdocipher_flutter/vdocipher_flutter.dart';

VideoPlayerPlatform? _lastVideoPlayerPlatform;

VideoPlayerPlatform get videoPlayerPlatform {
  final VideoPlayerPlatform currentInstance = VideoPlayerPlatform.instance;
  if (_lastVideoPlayerPlatform != currentInstance) {
    // This will clear all open videos on the platform when a full restart is
    // performed.
    currentInstance.init();
    _lastVideoPlayerPlatform = currentInstance;
  }
  return currentInstance;
}

typedef PlatformViewCreatedCallback = void Function(int id);

class _VideoAppLifeCycleObserver extends Object with WidgetsBindingObserver {
  _VideoAppLifeCycleObserver(this._controller);

  bool _wasPlayingBeforePause = false;
  final WebPlayerController _controller;

  void initialize() {
    _ambiguate(WidgetsBinding.instance)!.addObserver(this);
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    if (state == AppLifecycleState.paused) {
      _wasPlayingBeforePause = _controller.value.isPlaying;
      _controller.pause();
    } else if (state == AppLifecycleState.resumed) {
      if (_wasPlayingBeforePause) {
        _controller.play();
      }
    }
  }

  void dispose() {
    _ambiguate(WidgetsBinding.instance)!.removeObserver(this);
  }
}

/// Controls a platform video player, and provides updates when the state is
/// changing.
///
/// Instances must be initialized with initialize.
///
/// To reclaim the resources used by the player call [dispose].
///
/// After [dispose] all further calls are ignored.
class WebPlayerController extends VdoPlayerController {
  _VideoAppLifeCycleObserver? _lifeCycleObserver;
  StreamSubscription<dynamic>? _eventSubscription;
  final List<PlatformViewCreatedCallback> _platformViewCreatedCallbacks =
      <PlatformViewCreatedCallback>[];

  /// The id of a texture that hasn't been initialized.
  static const int kUninitializedTextureId = -1;
  int _textureId = kUninitializedTextureId;
  bool _isDisposed = false;

  WebPlayerController() : super();

  void addPlatformViewCreatedCallback(PlatformViewCreatedCallback listener) {
    if (!_isDisposed) {
      _platformViewCreatedCallbacks.add(listener);
    }
  }

  void removeOnPlatformViewCreatedListener(
      PlatformViewCreatedCallback listener) {
    if (!_isDisposed) {
      _platformViewCreatedCallbacks.remove(listener);
    }
  }

  int get textureId => _textureId;

  @override
  void init() {
    WidgetsBinding.instance.addObserver(this);
  }

  Future<void> initialize(EmbedInfo embedInfo, bool showControls) async {
    init();
    _lifeCycleObserver = _VideoAppLifeCycleObserver(this);
    _lifeCycleObserver?.initialize();

    _textureId = (await videoPlayerPlatform.create(embedInfo, showControls)) ??
        kUninitializedTextureId;

    for (PlatformViewCreatedCallback platformViewCreatedCallback
        in _platformViewCreatedCallbacks) {
      platformViewCreatedCallback(_textureId);
    }

    final Completer<void> initializingCompleter = Completer<void>();

    void eventListener(VideoEvent event) {
      if (_isDisposed) {
        return;
      }

      switch (event.eventType) {
        case VideoEventType.initialized:
          if (embedInfo.embedInfoOptions?.autoplay ?? false) {
            value.copyWith(isPlaying: true);
            notifyListeners();
            funStartPositionTimer?.call();
          }
          break;
        case VideoEventType.apiReady:
          break;
        case VideoEventType.play:
          value = value.copyWith(isPlaying: true, isBuffering: false);
          notifyListeners();
          break;
        case VideoEventType.playing:
          value = value.copyWithoutError(
              isPlaying: true,
              isBuffering: false,
              isLoading: false,
              isEnded: false);
          notifyListeners();
          break;
        case VideoEventType.waiting:
          value = value.copyWith(isBuffering: true, isPlaying: false);
          notifyListeners();
          break;
        case VideoEventType.pause:
          value = value.copyWith(isPlaying: false);
          notifyListeners();
          break;
        case VideoEventType.loadeddata:
          value = value.copyWith(isLoading: false);
          if (event.duration != null) {
            value = value.copyWith(duration: event.duration);
            notifyListeners();
          }
          _updateVideoTracks();
          _updateSubtitleTracks();
          _updateVolume();
          break;
        case VideoEventType.loadstart:
          videoPlayerPlatform.updatePlayerConfigMap(
              textureId, embedInfo.embedInfoOptions?.configMap);
          value = value.copyWith(isLoading: true);
          break;
        case VideoEventType.ratechange:
          if (event.playbackRate != null) {
            value = value.copyWith(playbackSpeed: event.playbackRate);
            notifyListeners();
          }
          break;
        case VideoEventType.progress:
          if (event.progress != null) {
            value = value.copyWith(position: event.progress);
            notifyListeners();
          }
          break;
        case VideoEventType.error:
          if (event.error != null) {
            Error error = event.error!;
            VdoError vdoError = VdoError(error.code!, error.message ?? "", -1);
            value = value.copyWith(vdoError: vdoError);
          }
          break;
        case VideoEventType.bufferingStart:
          break;
        case VideoEventType.bufferingEnd:
          break;
        case VideoEventType.bufferingUpdate:
          if (event.bufferedPosition != null) {
            value = value.copyWith(bufferedPosition: event.bufferedPosition);
            notifyListeners();
          }
          break;
        case VideoEventType.ended:
          value = value.copyWith(isEnded: true);
          notifyListeners();
          break;
        case VideoEventType.volumechange:
          if (event.volume != null) {
            value =
                value.copyWith(volume: event.volume, isMuted: event.isMuted);
            notifyListeners();
          }
          break;
        case VideoEventType.captionLanguageChange:
          if (event.textTrackId != null) {
            for (SubtitleTrack subtitleTrack in value.subtitleTracks) {
              if (subtitleTrack.id == event.textTrackId) {
                value = value.copyWith(subtitleTrack: subtitleTrack);
                break;
              }
            }
            notifyListeners();
          }
          break;
        case VideoEventType.videoQualityChange:
          if (event.videoTrackId != null) {
            for (VideoTrack videoTrack in value.videoTracks) {
              if (videoTrack.id == event.videoTrackId) {
                value = value.copyWith(videoTrack: videoTrack);
                break;
              }
            }
            notifyListeners();
          }
          break;
        case VideoEventType.videoAdaptivenessChange:
          value = value.copyWith(isAdaptive: event.isAdaptive);
          notifyListeners();
          break;
        case VideoEventType.captionVisibilityChange:
          if (!event.isCaptionEnabled) {
            value = value.copyWithDisabledTracks(subtitleTrack: null);
            notifyListeners();
          }
          break;
        case VideoEventType.unknown:
          break;
      }
    }

    void errorListener(Object obj) {
      final PlatformException e = obj as PlatformException;
      debugPrint("load error [ ${e.code} , ${e.message!} + ]");
      String errMsg = e.message ?? "unknown error";
      VdoError error = VdoError(int.parse(e.code), errMsg, null);
      value =
          value.copyWith(isLoading: false, isBuffering: false, vdoError: error);
    }

    _eventSubscription = videoPlayerPlatform
        .videoEventsFor(_textureId)
        .listen(eventListener, onError: errorListener);

    return initializingCompleter.future;
  }

  @override
  void dispose() {
    super.dispose();
    funStopPositionTimer?.call();
    disposeWebController();
  }

  Future<void> disposeWebController() async {
    if (_isDisposed) {
      return;
    }
    if (!_isDisposed) {
      _isDisposed = true;
      await _eventSubscription?.cancel();
      await videoPlayerPlatform.dispose(_textureId);
      _platformViewCreatedCallbacks.clear();
      WidgetsBinding.instance.removeObserver(this);
    }
    _lifeCycleObserver?.dispose();
    _isDisposed = true;
  }

  @override
  Future<void> load(EmbedInfo embedInfo) async {
    if (embedInfo.otp != null && embedInfo.playbackInfo != null) {
      await videoPlayerPlatform.load(
          textureId, embedInfo.otp!, embedInfo.playbackInfo!);
      if (embedInfo.embedInfoOptions?.autoplay ?? false) {
        value.copyWith(isPlaying: true);
        notifyListeners();
        if (value.isPlaying) {
          funStartPositionTimer?.call();
        }
      }
    }
  }

  @override
  Future<void> play() async {
    value = value.copyWith(isPlaying: true);
    funStartPositionTimer?.call();
    await videoPlayerPlatform.play(textureId);
    notifyListeners();
  }

  @override
  Future<void> pause() async {
    value = value.copyWith(isPlaying: false);
    await videoPlayerPlatform.pause(textureId);
    notifyListeners();
  }

  @override
  Future<void> stop() async {}

  @override
  Future<Duration> getDuration() async {
    return videoPlayerPlatform.getDuration(textureId);
  }

  @override
  Future<Duration> getPosition() async {
    return videoPlayerPlatform.getPosition(textureId);
  }

  @override
  Future<Duration> getBufferedPosition() {
    return videoPlayerPlatform.getBufferedPosition(textureId);
  }

  @override
  Future<void> setVolume(double volume) {
    return videoPlayerPlatform.setVolume(textureId, volume);
  }

  @override
  Future<void> setMute(bool mute) {
    return videoPlayerPlatform.setMute(textureId, mute);
  }

  @override
  Future<void> seek(Duration target) async {
    target = target > value.duration ? value.duration : target;
    return videoPlayerPlatform.seekTo(textureId, target);
  }

  @override
  Future<void> setPlaybackSpeed(double speed) async {
    value = value.copyWith(playbackSpeed: speed);
    await videoPlayerPlatform.setPlaybackSpeed(textureId, 2);
    notifyListeners();
  }

  @override
  Future<void> setAdaptive() async {
    return videoPlayerPlatform.setAdaptive(textureId);
  }

  @override
  Future<Object> getPlaybackProperty(String propertyName) {
    return videoPlayerPlatform.getPlaybackProperty(textureId, propertyName);
  }

  void _updateVideoTracks() async {
    List<VideoTrack> videoTracks =
        await videoPlayerPlatform.getVideoTracks(textureId);
    VideoTrack? videoTrack = await videoPlayerPlatform.getVideoTrack(textureId);
    value = value.copyWith(videoTracks: videoTracks, videoTrack: videoTrack);
    notifyListeners();
  }

  void _updateSubtitleTracks() async {
    List<SubtitleTrack> subtitleTracks =
        await videoPlayerPlatform.getSubtitleTracks(textureId);
    SubtitleTrack? subtitleTrack =
        await videoPlayerPlatform.getSubtitleTrack(textureId);
    value = value.copyWith(
        subtitleTracks: subtitleTracks, subtitleTrack: subtitleTrack);
    notifyListeners();
  }

  void _updateVolume() async {
    double volume = await videoPlayerPlatform.getVolume(textureId);
    value = value.copyWith(volume: volume);
    notifyListeners();
  }

  @override
  Future<void> setVideoTrack(VideoTrack videoTrack) async {
    return videoPlayerPlatform.setVideoTrackId(textureId, videoTrack.id);
  }

  @override
  Future<void> setSubtitleLanguage(String? language) async {
    return videoPlayerPlatform.setTextTrackLanguage(textureId, language);
  }
}

T? _ambiguate<T>(T? value) => value;
